//+------------------------------------------------------------------+
//|                                                  MACD Sample.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+

extern double TakeProfit = 50;
extern double Lots = 0.1;
extern double TrailingStop = 30;
extern double MACDOpenLevel=3;
extern double MACDCloseLevel=2;
extern double MATrendPeriod=26;

//+------------------------------------------------------------------+
//| helper functions                                                 |
//+------------------------------------------------------------------+

int sign( double v )
{
    if( v < 0 ) return( -1 );
    return( 1 );
}

//+------------------------------------------------------------------+
double iif( bool condition, double ifTrue, double ifFalse )
{
    if( condition ) return( ifTrue );
    
    return( ifFalse );
}

//+------------------------------------------------------------------+
string iifStr( bool condition, string ifTrue, string ifFalse )
{
    if( condition ) return( ifTrue );

    return( ifFalse );
}

//+------------------------------------------------------------------+
int orderDirection()
{
    return( 1 - 2 * ( OrderType() % 2 ) );
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   double MacdCurrent, MacdPrevious, SignalCurrent;
   double SignalPrevious, MaCurrent, MaPrevious;
   int cnt, ticket, total;
// initial data checks
// it is important to make sure that the expert works with a normal
// chart and the user did not make any mistakes setting external 
// variables (Lots, StopLoss, TakeProfit, 
// TrailingStop) in our case, we check TakeProfit
// on a chart of less than 100 bars
   if(Bars<100)
     {
      Print("bars less than 100");
      return(0);  
     }
   if(TakeProfit<10)
     {
      Print("TakeProfit less than 10");
      return(0);  // check TakeProfit
     }
// to simplify the coding and speed up access
// data are put into internal variables
   MacdCurrent=iMACD(NULL,0,12,26,9,PRICE_CLOSE,MODE_MAIN,0);
   MacdPrevious=iMACD(NULL,0,12,26,9,PRICE_CLOSE,MODE_MAIN,1);
   SignalCurrent=iMACD(NULL,0,12,26,9,PRICE_CLOSE,MODE_SIGNAL,0);
   SignalPrevious=iMACD(NULL,0,12,26,9,PRICE_CLOSE,MODE_SIGNAL,1);
   MaCurrent=iMA(NULL,0,MATrendPeriod,0,MODE_EMA,PRICE_CLOSE,0);
   MaPrevious=iMA(NULL,0,MATrendPeriod,0,MODE_EMA,PRICE_CLOSE,1);

   total=OrdersTotal();
   if(total<1) 
     {
      // no opened orders identified
      if(AccountFreeMargin()<(1000*Lots))
        {
         Print("We have no money. Free Margin = ", AccountFreeMargin());
         return(0);  
        }
      
      int tradeDirection = -sign( MacdCurrent );
      
      // check if we can enter the market
      if( MacdCurrent * tradeDirection < 0 && ( MacdCurrent - SignalCurrent ) * tradeDirection > 0 
         && ( MacdPrevious - SignalPrevious ) * tradeDirection < 0 && MathAbs(MacdCurrent)>(MACDOpenLevel*Point)
         && ( MaCurrent - MaPrevious ) * tradeDirection > 0 )
        {
         int orderType = iif( tradeDirection > 0, OP_BUY, OP_SELL );
         string orderTypeName = iifStr( tradeDirection > 0, "BUY", "SELL" );
         double openPrice = iif( tradeDirection > 0, Ask, Bid );
         color c = iif( tradeDirection > 0, Green, Red );
         ticket = OrderSend( Symbol(), orderType, Lots, openPrice, 3 , 0, openPrice + tradeDirection * TakeProfit * Point, 
            "macd sample", 16384, 0, c );
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print( orderTypeName + " order opened : ", OrderOpenPrice() );
           }
         else Print("Error opening " + orderTypeName + " order : ",GetLastError()); 
         return(0); 
        }
      return(0);
     }
   // it is important to enter the market correctly, 
   // but it is more important to exit it correctly...   
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL &&   // check for opened position 
         OrderSymbol()==Symbol())  // check for symbol
        {
         tradeDirection = orderDirection();
         double closePrice = iif( tradeDirection > 0, Bid, Ask );
         c = iif( tradeDirection > 0, Green, Red );
         
         // should it be closed?
         if( MacdCurrent * tradeDirection > 0 && ( MacdCurrent - SignalCurrent ) * tradeDirection < 0 
            && ( MacdPrevious - SignalPrevious ) * tradeDirection > 0 
            && MathAbs( MacdCurrent ) > ( MACDCloseLevel * Point ) )
             {
              OrderClose(OrderTicket(),OrderLots(), closePrice, 3,Violet); // close position
              return(0); // exit
             }
         // check for trailing stop
         if(TrailingStop>0)  
           {                 
            if( ( closePrice - OrderOpenPrice() ) * tradeDirection > Point * TrailingStop )
              {
               if( OrderStopLoss() == 0 || ( OrderStopLoss() - ( closePrice - tradeDirection * Point * TrailingStop ) ) * 
                 tradeDirection < 0 )
                 {
                  OrderModify( OrderTicket(), OrderOpenPrice(), closePrice - tradeDirection * Point * TrailingStop, 
                     OrderTakeProfit(), 0, c );
                  return(0);
                 }
              }
           }
         }
     }
   return(0);
  }
// the end.
